/*-
 * Copyright (C) @BABOLO  2002 Dec 04
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char copyright[] = "\
@(#)Copyright (C) @BABOLO  2002 Dec 04\n\
@(#)All rights reserved.\n";
static const char rcsid[] = "$Id: pgoblin.c,v 1.7 2004/11/05 02:21:02 babolo Exp $";
#endif /* not lint */

#include <sys/types.h>
#include <sysexits.h>
#include <unistd.h>
#include <stdlib.h>
#include <limits.h>
#include <string.h>
#include <stdio.h>
#include <mife.h>
#include <err.h>
#include <libpq-fe.h>
#include "pgoblin.h"

struct tabdelim tdl = {NULL, NULL, NULL, NULL, NULL, NULL};

void
usage(int ex) {
    fprintf(stderr, "pgoblin " VERS "  " DATE "
Usage:
pgoblin -[BDqsSv ][-c Table ][-C Table ][-d DataBase ][-h Hostname ][-p Port ][-U User ][query file]
    -d DataBase name
    -h Hostname of backend server
    -p Port
____-U Username
    -c COPY out
    -C COPY in
    -D perform query without result
    -s SELECT
____-S SELECT BINARY emulation
    -q quiet
    -v verbose
");
    exit(ex);
}

int
main(int argc, char *argv[]) {
    struct pgoblincon *options = malloc(sizeof(struct pgoblincon));
    struct pgoblin *pgm;
    int out = fileno(stdout);
    struct mife_descriptor *in, *pip;
    char *query;
    int c;

    query = NULL;
    if  (options == NULL) errx(EX_UNAVAILABLE, "Malloc failed #1\n");
    bzero(options, sizeof(struct pgoblincon));
    options->cmd = pgob_null;
    options->tdl = &tdl;
    options->flag = PGOB_VERB;
    while ((c = getopt(argc, argv, "Bc:C:d:D:h:p:qs:S:U:v?")) != -1) {
        switch (c) {
        case 'B': break;
        case 'c': if (options->cmd != pgob_null) usage(EX_USAGE);
                  options->cmd = pgob_copyout;
                  query = optarg;
                  break;
        case 'C': if (options->cmd != pgob_null) usage(EX_USAGE);
                  options->cmd = pgob_copyin;
                  query = optarg;
                  break;
        case 'd': options->dbname = optarg;
                  break;
        case 'D': if (options->cmd != pgob_null) usage(EX_USAGE);
                  options->cmd = pgob_perform;
                  query = optarg;
                  break;
        case 'h': options->host = optarg;
                  break;
        case 'p': options->port = optarg;
                  break;
        case 'q': options->flag &= ~PGOB_VERMASK;
                  break;
        case 's': if (options->cmd != pgob_null) usage(EX_USAGE);
                  options->cmd = pgob_strselect;
                  query = optarg;
                  break;
        case 'S': if (options->cmd != pgob_null) usage(EX_USAGE);
                  options->cmd = pgob_binselect;
                  query = optarg;
                  break;
        case 'U': options->username = optarg;
                  break;
        case 'v': options->flag |= ((options->flag << 1) & PGOB_VERMASK) | PGOB_VERB;
                  break;
        case '?': usage(EX_OK);
        default:  usage(EX_USAGE);
    }   }
    argc -= optind;
    argv += optind;
    if  (argc > 0 && argv && *argv) {
        if  (!(in = mife_open(MIFE_FULL, *argv))) err(EX_NOINPUT, "file %s:", *argv);
    } else {
        if  (!(in = mife_opef(MIFE_FULL, fileno(stdin)))) err(EX_NOINPUT, "stdin:");
    }
    /* Prepare the query */
    switch (options->cmd) {
    case pgob_null:
        if  ((pgm = pgoblin_parser(options, in)) == NULL) {
            if (options->flag & PGOB_VERB) errx(EX_DATAERR, "Parser failed");
            exit(EX_DATAERR);
        }
        if  (in->fd == fileno(stdin)) pip = in;
        else  if  (!(pip = mife_opef(MIFE_FULL, fileno(stdin)))) err(EX_NOINPUT, "stdin:");

        if  (pgoblin_execute(options, pgm, pip, out)) {
            if (options->flag & PGOB_VERB) errx(EX_DATAERR, "Execute failed");
            exit(EX_DATAERR);
        }
        break;
    case pgob_copyin:
        if  (pgoblin_copyin(options, query, in)) {
            if (options->flag & PGOB_VERB) errx(EX_DATAERR, "COPY in failed");
            exit(EX_DATAERR);
        }
        break;
    case pgob_copyout:
        if  (pgoblin_copyout(options, query, out)) {
            if (options->flag & PGOB_VERB) errx(EX_DATAERR, "COPY out failed");
            exit(EX_DATAERR);
        }
        break;
    case pgob_binselect:
        if  (pgoblin_binsel(options, query, out)) {
            if (options->flag & PGOB_VERB) errx(EX_DATAERR, "SELECT BINARY failed");
            exit(EX_DATAERR);
        }
        break;
    case pgob_strselect:
        if  (pgoblin_strsel_p(options, pgoblin_strsel_q(options, query), out)) {
            if (options->flag & PGOB_VERB) errx(EX_DATAERR, "SELECT failed");
            exit(EX_DATAERR);
        }
        break;
    case pgob_perform:
        if  (pgoblin_perform(options, query)) {
            if (options->flag & PGOB_VERB) errx(EX_DATAERR, "perform failed");
            exit(EX_DATAERR);
        }
        break;
    default:
        usage(EX_USAGE);
    }
    if  (options->conn) PQfinish(options->conn);
    exit(EX_OK);
}
